### 本程序的技术博客
### http://blog.mangolovecarrot.net/2025/01/19/pico-02

from machine import Pin, SPI, I2C
from max7219 import max7219
import time
import random

# SPI接口设置
spi = SPI(1, baudrate=10000000, polarity=0, phase=0)
cs = Pin(0, Pin.OUT)

# I2C接口设置 (修改为I2C1，SDA=GPIO2, SCL=GPIO3)
i2c = I2C(1, scl=Pin(3), sda=Pin(2), freq=400000)

def scan_i2c():
    devices = i2c.scan()
    if len(devices) == 0:
        print("没有检测到I2C设备。")
        return None
    elif len(devices) == 1:
        print(f"检测到1个设备，地址: 0x{devices[0]:02X}")
        return devices[0]
    else:
        print("检测到多个I2C设备:")
        for i, device in enumerate(devices):
            print(f"{i + 1}: 0x{device:02X}")
        while True:
            try:
                choice = int(input("请选择设备 (输入序号): ")) - 1
                if 0 <= choice < len(devices):
                    return devices[choice]
                else:
                    print("无效选择，请重试。")
            except ValueError:
                print("请输入有效数字。")

EEPROM_ADDR = scan_i2c()
if EEPROM_ADDR is None:
    raise Exception("未检测到EEPROM设备，程序终止。")

EEPROM_MEM_ADDR = 0x0000  # 指定写入和读取的地址

# 初始化MAX7219
matrix = max7219.Matrix8x8(spi, cs, 1)
matrix.brightness(0)

# 显示方向（1=0°，2=90°，3=180°，4=270°）
display_direction = 2  # 默认顺时针旋转90度

# 定义数字0-9、问号和叉的图案
digits = [
    [0x3E, 0x63, 0x67, 0x6F, 0x7B, 0x73, 0x63, 0x3E],  # 0
    [0x0C, 0x1C, 0x3C, 0x0C, 0x0C, 0x0C, 0x0C, 0x3F],  # 1
    [0x3E, 0x63, 0x03, 0x0E, 0x3C, 0x70, 0x63, 0x7F],  # 2
    [0x3E, 0x63, 0x03, 0x1E, 0x03, 0x03, 0x63, 0x3E],  # 3
    [0x06, 0x0E, 0x1E, 0x36, 0x66, 0x7F, 0x06, 0x06],  # 4
    [0x7F, 0x60, 0x7E, 0x03, 0x03, 0x03, 0x63, 0x3E],  # 5
    [0x1E, 0x30, 0x60, 0x7E, 0x63, 0x63, 0x63, 0x3E],  # 6
    [0x7F, 0x63, 0x06, 0x0C, 0x18, 0x18, 0x18, 0x18],  # 7
    [0x3E, 0x63, 0x63, 0x3E, 0x63, 0x63, 0x63, 0x3E],  # 8
    [0x3E, 0x63, 0x63, 0x63, 0x3F, 0x03, 0x06, 0x3C]   # 9
]
question_mark = [0x3E, 0x63, 0x03, 0x0E, 0x1C, 0x00, 0x1C, 0x1C]  # ?
cross_mark = [0x63, 0x36, 0x1C, 0x08, 0x1C, 0x36, 0x63, 0x00]    # X

def rotate_pattern(pattern, direction):
    rotated = [[0] * 8 for _ in range(8)]
    for y in range(8):
        for x in range(8):
            bit = (pattern[y] >> (7 - x)) & 1
            if direction == 1:
                rotated[y][x] = bit
            elif direction == 2:
                rotated[x][7 - y] = bit
            elif direction == 3:
                rotated[7 - y][7 - x] = bit
            elif direction == 4:
                rotated[7 - x][y] = bit
    return [int("".join(str(bit) for bit in row), 2) for row in rotated]

def display_pattern(pattern):
    rotated = rotate_pattern(pattern, display_direction)
    matrix.fill(0)
    for y in range(8):
        matrix.buffer[y] = rotated[y]
    matrix.show()

def write_eeprom(addr, value):
    addr_high = (addr >> 8) & 0xFF
    addr_low = addr & 0xFF
    i2c.writeto(EEPROM_ADDR, bytes([addr_high, addr_low, value]))
    time.sleep(0.01)

def read_eeprom(addr):
    addr_high = (addr >> 8) & 0xFF
    addr_low = addr & 0xFF
    i2c.writeto(EEPROM_ADDR, bytes([addr_high, addr_low]))
    return int.from_bytes(i2c.readfrom(EEPROM_ADDR, 1), 'big')

last_number = -1

while True:
    new_number = random.choice([i for i in range(10) if i != last_number])
    last_number = new_number
    write_eeprom(EEPROM_MEM_ADDR, new_number)
    display_pattern(question_mark)
    time.sleep(0.5)
    read_number = read_eeprom(EEPROM_MEM_ADDR)
    if read_number == new_number:
        display_pattern(digits[read_number])
    else:
        display_pattern(cross_mark)
    time.sleep(1)

